package org.expeditee.agents;

import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import org.expeditee.greenstone.Greenstone3Connection;
import org.expeditee.greenstone.Query;
import org.expeditee.greenstone.QueryOutcome;
import org.expeditee.greenstone.Result;
import org.expeditee.greenstone.ResultDocument;
import org.expeditee.gui.AttributeValuePair;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameCreator;
import org.expeditee.gui.FrameGraphics;
import org.expeditee.items.Text;

public class SearchGreenstone extends SearchAgent {

	private static boolean _doCasefolding = true;

	private static boolean _doStemming = false;

	protected static Greenstone3Connection _gsdl = null;

	private static String _maxResults = "10";

	private int _indexChoice = 1;

	private static boolean _getAllMetadata = true;

	private static int _locationChoice = 1;

	private static String[] _indexKeys = { "TX", "TI", "JO", "BO", "CR", "KE" };

	protected static Vector<Result> _currentResultSet;

	public SearchGreenstone(String searchText) {
		super(searchText);
	}

	public SearchGreenstone(int resultsCount, String searchText) {
		this(searchText);
		_maxResults = resultsCount + "";
	}

	public static void init(Frame settings) {
		if (settings == null)
			return;

		// Set the settings
		for (Text item : settings.getBodyTextItems(false)) {

			AttributeValuePair avp = new AttributeValuePair(item.getText());
			if (avp.isAnnotation())
				continue;

			String attribute = avp.getAttributeOrValue().toLowerCase();

			if (attribute.equals("campus"))
				_locationChoice = 0;
			if (attribute.equals("autoconnect"))
				connect();

			if (attribute.equals("maxresults")) {
				try {
					_maxResults = avp.getValue();
				} catch (Exception e) {
				}
			}

			if (attribute.equals("dostemming"))
				_doStemming = true;
		}
	}

	public static void connect() {
		if (_gsdl == null)
			_gsdl = new Greenstone3Connection(_locationChoice);
	}

	@Override
	protected Frame process(Frame frame) {
		_results.setTitle("SearchGreenstone [" + _pattern + "]");

		connect();

		doQuery(_pattern);

		createResults();
		
		_results.save();

		String resultFrameName = _results.getName();
		if (_clicked != null)
			_clicked.setLink(resultFrameName);

		return _results.getFirstFrame();
	}

	protected void createResults() {
		viewByScore(_currentResultSet, _results);
	}

	protected void doQuery(String queryText) {
		Query query = createQuery(queryText);
		QueryOutcome queryOutcome = _gsdl.issueQueryToServer(query);
		_currentResultSet = getResultSetMetadata(queryOutcome);
	}

	private Query createQuery(String queryText) {

		Query query = new Query();

		// set the query options
		query.setQueryText(queryText);

		query.setIndex(_indexKeys[_indexChoice]);
		query.setMaxDocsToReturn(_maxResults);

		if (_doStemming) {
			query.setStemming("1");
		} else {
			query.setStemming("0");
		}

		if (_doCasefolding) {
			query.setCasefolding("1");
		} else {
			query.setCasefolding("0");
		}

		return query;
	}

	public Vector<Result> getResultSetMetadata(QueryOutcome queryOutcome) {

		Vector<Result> results = queryOutcome.getResults();
		for (Enumeration<Result> e = results.elements(); e.hasMoreElements();) {
			Result result = e.nextElement();
			getResultMetadata(result);
		}
		return results;
	}

	private void getResultMetadata(Result result) {
		String docID = result.getDocID();

		_gsdl.getDocumentMetadataFromServer(docID, "Title");
		_gsdl.getDocumentMetadataFromServer(docID, "Date");

		if (_getAllMetadata) {
			_gsdl.getDocumentMetadataFromServer(docID, "Date");
			_gsdl.getDocumentMetadataFromServer(docID, "Booktitle");
			_gsdl.getDocumentMetadataFromServer(docID, "Journal");
			_gsdl.getDocumentMetadataFromServer(docID, "Creator");
			_gsdl.getDocumentMetadataFromServer(docID, "Keywords");
			_gsdl.getDocumentMetadataFromServer(docID, "Publisher");
			_gsdl.getDocumentMetadataFromServer(docID, "Abstract");
			_gsdl.getDocumentMetadataFromServer(docID, "Pages");
			_gsdl.getDocumentMetadataFromServer(docID, "Number");
			_gsdl.getDocumentMetadataFromServer(docID, "Volume");
		}

	}

	/*
	 * given the Vector of result items (ordered by descending relevance to the
	 * query) this method iterates through them and constructs an HTML document
	 * and has it rendered in the result window
	 * 
	 * This is the default presentation for results
	 * 
	 * You can modify this method as you wish to change the HTML for the default
	 * presentation
	 * 
	 * Some useful method calls are illustrated here - if you have the ID of a
	 * document you can get all data stored for it with ResultDocument rd =
	 * gsdl.getDocument(docID); - a ResultDocument object has a set of methods
	 * for getting metadata values for that document most metadata is a single
	 * value, but authors and keywords can have multiple values and these are
	 * stored in a Vector
	 * 
	 * The IF condition for 'Title' shows how to construct a link that can be
	 * clicked on and then dealt with by the handleLinkClick() method
	 * 
	 */
	public void viewByScore(Vector<Result> results, FrameCreator resultsCreator) {

		for (Result result : results) {
			String docID = result.getDocID();
			ResultDocument rd = _gsdl.getDocument(docID);
			int docRank = result.getRank();
			// Put the details on a separate frame
			FrameCreator details = new FrameCreator(rd.getTitle());
			details.addText(getDetails(rd), null, null, null, true);

			resultsCreator.addText((docRank + 1) + ". " + rd.getTitle(), null,
					details.getName(), "getTextFromChildFrame", false);

			FrameGraphics.requestRefresh(true);
		}
	}

	protected String getDetails(ResultDocument rd) {
		StringBuffer resultText = new StringBuffer("");

		if (rd.metadataExists("Title")) {
			resultText.append("title = " + rd.getTitle()).append('\n');
		}
		if (rd.metadataExists("Date")) {
			resultText.append("date = " + rd.getDate()).append('\n');
		}
		if (rd.metadataExists("Booktitle")) {
			resultText.append("booktitle = " + rd.getBooktitle()).append('\n');
		}
		if (rd.metadataExists("Pages")) {
			resultText.append("pages = " + rd.getPages()).append('\n');
		}
		if (rd.metadataExists("Journal")) {
			resultText.append("journal = " + rd.getJournal()).append('\n');
		}
		if (rd.metadataExists("Volume")) {
			resultText.append("volume = " + rd.getVolume()).append('\n');
		}
		if (rd.metadataExists("Number")) {
			resultText.append("number = " + rd.getNumber()).append('\n');
		}
		if (rd.metadataExists("Editor")) {
			resultText.append("editor = " + rd.getEditor()).append('\n');
		}
		if (rd.metadataExists("Publisher")) {
			resultText.append("publisher = " + rd.getPublisher()).append('\n');
		}
		if (rd.metadataExists("Abstract")) {
			resultText.append("abstract = " + rd.getAbstract()).append('\n');
		}
		Vector<String> authors = rd.getAuthors();
		for (Enumeration a = authors.elements(); a.hasMoreElements();) {
			String author = (String) a.nextElement();
			resultText.append("author = " + author).append('\n');
		}
		Vector<String> keywords = rd.getKeywords();
		for (Enumeration k = keywords.elements(); k.hasMoreElements();) {
			String keyword = (String) k.nextElement();
			resultText.append("keyword = " + keyword).append('\n');
		}
		resultText.deleteCharAt(resultText.length() - 1);

		return resultText.toString();
	}

}
