package org.expeditee.agents;

import java.util.Collections;
import java.util.HashMap;
import java.util.Vector;

import org.expeditee.greenstone.Result;
import org.expeditee.greenstone.ResultDocument;
import org.expeditee.gui.FrameCreator;
import org.expeditee.gui.FrameGraphics;

public class SearchGreenstoneByDate extends SearchGreenstone {

	public SearchGreenstoneByDate(String searchText) {
		super(searchText);
	}

	public SearchGreenstoneByDate(int resultsCount, String searchText) {
		super(resultsCount, searchText);
	}

	/**
	 * dateMap is a hash table. The keys are year values. the data associated
	 * with each key is a Vector of document IDs therefore, for the current
	 * result set you can get the set of years in which the results were
	 * published, and for each year you can get the set of documents published
	 * in that year
	 * 
	 * If you want to introduce additional mappings (eg document written by
	 * authors) you should introduce additional structures here (HashMap used in
	 * the same way as dateMap will probably suffice
	 * 
	 */
	private static HashMap<String, Vector<String>> _dateMap = new HashMap<String, Vector<String>>();

	protected void doQuery(String queryText) {
		_dateMap.clear();

		super.doQuery(queryText);

		initialiseDateMap(_currentResultSet);
	}

	protected void createResults() {
		viewByDate(_dateMap, _results);
	}

	public void initialiseDateMap(Vector<Result> results) {
		for (Result result : results) {
			String docID = result.getDocID();

			ResultDocument rd = _gsdl.getDocument(docID);

			if (rd.metadataExists("Date")) {
				addToDateMap(docID, rd);
			}
		}
	}

	public void addToDateMap(String docID, ResultDocument rd) {
		if (_dateMap.containsKey(rd.getDate())) {
			Vector<String> dateVector = _dateMap.get(rd.getDate());
			if (!dateVector.contains(docID)) {
				dateVector.addElement(docID);
			}
			_dateMap.put(rd.getDate(), dateVector);
		} else {
			Vector<String> dateVector = new Vector<String>();
			dateVector.addElement(docID);
			_dateMap.put(rd.getDate(), dateVector);
		}
	}

	/**
	 * This method provides an alternative view of the result set It uses the
	 * dateMap which organises results according to the year in which they were
	 * published
	 * 
	 * You will need to write a similar method for any additional view that you
	 * implement
	 * 
	 * You may wish to modify this method to provide a better layout of the
	 * output
	 */
	private void viewByDate(HashMap<String, Vector<String>> dateMap,
			FrameCreator results) {
		Vector<String> theDates = new Vector<String>(dateMap.keySet());
		Collections.sort(theDates);

		for (String date : theDates) {
			Vector<String> ids = dateMap.get(date);

			// Text dateText = results.addText(date, null, null, null, false);
			// dateText.setFontStyle("bold");

			for (String docID : ids) {
				ResultDocument rd = _gsdl.getDocument(docID);

				// Put the details on a separate frame
				FrameCreator details = new FrameCreator(rd.getTitle());
				details.addText(getDetails(rd), null, null, null, true);

				results.addText(date + "- " + rd.getTitle(), null, details
						.getName(), "getTextFromChildFrame", false);

				FrameGraphics.requestRefresh(true);
			}
		}
	}
}
